import requests

CATBOX_API = 'https://catbox.moe/user/api.php'

class AlbumManager:
    def __init__(self, userhash: str | None = None):
        self.userhash = userhash


    def _post(self, data: dict) ->str:
        if self.userhash:
            data["userhash"] = self.userhash
        response = requests.post(CATBOX_API, data = data)
        if response.status_code == 200:
            return response.text.strip()
        
        else:
            raise Exception(f'Request Failed: {response.status_code} {response.text}')
        
    def create(self,title: str, desc:str, files : list[str]) ->str:
        '''
        Create a new album
        Params: 
        Album Title,
        Album Description,
        List of catbox file urls'''

        return self._post({
            'reqtype' :'createalbum',
            'title' : title,
            'desc' : desc,
            'files' : " ".join(files),

        })
    def _check_userhash(self):
        if not self.userhash:
            raise ValueError("This operation requires a userhash (logged in user)")
    def edit(self,short: str, title: str, desc: str, files: list[str]) ->str:
        '''
        Editing an Album
        Params:
        Short: 6 alphanumeric characters in the url thats generated,
        title: Album Title,
        desc: Album description,
        Files: List of file urls
        '''
        self._check_userhash()
        return self._post({
            'reqtype' :'createalbum',
            'short' : short,
            'title' : title,
            'desc' : desc,
            'files' : " ".join(files),

        })
    
    def add_files(self, short: str, files: list[str]) -> str:
        """
        Add files to an existing album.

        short: Album ID
        files: Files to add
        """
        self._check_userhash()
        return self._post({
            "reqtype": "addtoalbum",
            "short": short,
            "files": " ".join(files),
        })

    def remove_files(self, short: str, files: list[str]) -> str:
        """
        removes files from an album.

        short: Album ID
        files: Files to remove

        """
        self._check_userhash()
        return self._post({
            "reqtype": "removefromalbum",
            "short": short,
            "files": " ".join(files),
        })

    def delete(self, short: str) -> str:
        """
        deletes an album.

        short: Album ID
        """
        self._check_userhash()
        return self._post({
            "reqtype": "deletealbum",
            "short": short,
        })
    
class AnonymousAlbumProxy:
    def __getattr__(self, name):
        raise RuntimeError("User Hash is required. Anonymous clients cannot manage albums.")



import pytest
from catboxpy import LitterboxClient

@pytest.mark.asyncio
async def test_upload_file():
    client = LitterboxClient()
    url = await client.upload("tests/example.jpg", expire_time="1h")
    assert url.startswith("https://files.catbox.moe/")


import os
import pytest
from catboxpy import CatboxClient

client  = CatboxClient()

def test_upload_url():
    url = client.upload('https://docs.pytest.org/en/stable/_static/pytest1.png')
    assert url.startswith('https://files.catbox.moe/')


@pytest.mark.skipif(not os.path.exists("tests/test.png"), reason="Local file not found")
def test_upload_file():
    url = client.upload("tests/example.jpg")
    assert url.startswith("https://files.catbox.moe/")


import httpx

CATBOX_API = "https://catbox.moe/user/api.php"

class AsyncAlbumManager:
    def __init__(self, userhash: str | None = None):
        self.userhash = userhash

    async def _post(self, data: dict) -> str:
        if self.userhash:
            data["userhash"] = self.userhash
        async with httpx.AsyncClient() as client:
            response = await client.post(CATBOX_API, data=data)
        if response.status_code == 200:
            return response.text.strip()
        else:
            raise Exception(f"Request failed: {response.status_code} {response.text}")
        
    async def _check_userhash(self):
        if not self.userhash:
            raise ValueError("This operation requires a userhash (logged in user)")
        
    async def create(self, title: str, desc: str, files: list[str]) -> str:

        return await self._post({
            "reqtype": "createalbum",
            "title": title,
            "desc": desc,
            "files": " ".join(files),
        })

    async def edit(self, short: str, title: str, desc: str, files: list[str]) -> str:
        await self._check_userhash()
        return await self._post({
            "reqtype": "editalbum",
            "short": short,
            "title": title,
            "desc": desc,
            "files": " ".join(files),
        })

    async def add_files(self, short: str, files: list[str]) -> str:
        await self._check_userhash()
        return await self._post({
            "reqtype": "addtoalbum",
            "short": short,
            "files": " ".join(files),
        })

    async def remove_files(self, short: str, files: list[str]) -> str:
        await self._check_userhash()
        return await self._post({
            "reqtype": "removefromalbum",
            "short": short,
            "files": " ".join(files),
        })

    async def delete(self, short: str) -> str:
        await self._check_userhash()
        return await self._post({
            "reqtype": "deletealbum",
            "short": short,
        })


import pytest
from catboxpy import AsyncCatboxClient

@pytest.mark.asyncio
async def test_upload_url_async():
    client = AsyncCatboxClient()
    url = await client.upload("tests/example.jpg")
    assert url.startswith("https://files.catbox.moe/")



import requests


class LitterboxClient:

    LITTERBOX_API = "https://litterbox.catbox.moe/resources/internals/api.php"
    VALID_TIMES = ["1h", "12h", "24h", "72h"]

    def __init__(self):
        pass

    def upload_file(self, filepath: str, expire_time: str = "24h") -> str:
        """
        Upload a file to Litterbox.
        
        :param filepath: Path to the file to upload
        :param expire_time: Time before the file expires ('1h', '12h', '24h', or '72h')
        :return: URL to the uploaded file
        :raises: ValueError if expire_time is invalid, or requests.exceptions.RequestException on network errors
        """

        if expire_time not in self.VALID_TIMES:
            raise ValueError(f"Invalid expire_time '{expire_time}'. Must be one of {self.VALID_TIMES}")
        
        with open(filepath, "rb") as f:
            files = {
                "fileToUpload": f
            }
            data = {
                "reqtype": "fileupload",
                "time": expire_time
            }

            response = requests.post(self.LITTERBOX_API, files=files, data=data)
            response.raise_for_status()

            if response.text.startswith("https://"):
                return response.text.strip()
            else:
                raise RuntimeError(f"Unexpected response from Litterbox: {response.text}")


import requests
from .album import AlbumManager

CATBOX_API = 'https://catbox.moe/user/api.php'


class CatboxClient:

    def __init__(self, userhash: str | None = None):

        self.userhash = userhash
        self.album = AlbumManager(userhash=self.userhash) 
        

    def upload(self,file_or_url: str) ->str:
        '''Upload an File to Catbox
            by uploading a file or by using an Url.'''
        
        if file_or_url.startswith('https://') or file_or_url.startswith('http://'):
            return self.url_upload(file_or_url)
        
        else:
            return self.file_upload(file_or_url)
    
    def file_upload(self, file_path: str):
        data={
            'reqtype':'fileupload',
            }  
        if self.userhash:
            data["userhash"] = self.userhash
        with open(file_path, 'rb') as f:
            files = {'fileToUpload' : f}
            response = requests.post(CATBOX_API, data=data, files=files)
        
        if response.status_code==200:
            return response.text.strip()
        else:
            raise Exception(f'Failed to Upload File: {response.status_code} {response.text}')
        


    def url_upload(self, file_url: str) -> str:
        data = {
            "reqtype": "urlupload",
            "url": file_url,
        }
        if self.userhash:
            data["userhash"] = self.userhash

        response = requests.post(CATBOX_API, data=data)
        if response.status_code == 200:
            return response.text.strip()
        else:
            raise Exception(f"URL upload failed: {response.status_code} {response.text}")

        

    def delete_file(self, filename: str) -> str:
        if not self.userhash:
            raise ValueError("Userhash is required to delete files")

        data = {
            "reqtype": "deletefiles",
            "userhash": self.userhash,
            "files": filename  
        }
        res = requests.post(CATBOX_API, data=data)
        return res.text.strip()




import httpx
import os
from .async_album import AsyncAlbumManager

CATBOX_API = 'https://catbox.moe/user/api.php'


class AsyncCatboxClient:

    def __init__(self, userhash: str | None = None):

        self.userhash = userhash
        self.album = AsyncAlbumManager(userhash=self.userhash) 
        

    async def upload(self,file_or_url: str) ->str:
        '''Upload an File to Catbox
            by uploading a file or by using an Url.'''
        
        if file_or_url.startswith('https://') or file_or_url.startswith('http://'):
            return await self._upload_url(file_or_url)
        
        else:
            return await self._upload_file(file_or_url)
    
    async def _upload_file(self, file_path: str) -> str:
        data = {'reqtype': 'fileupload'}
        if self.userhash:
            data['userhash'] = self.userhash

        async with httpx.AsyncClient() as client:
            with open(file_path, 'rb') as f:
                files = {'fileToUpload': (os.path.basename(file_path), f, 'application/octet-stream')}
                response = await client.post(CATBOX_API, data=data, files=files)

        if response.status_code == 200:
            return response.text.strip()
        else:
            raise Exception(f"File upload failed: {response.status_code} {response.text}")
        


    async def _upload_url(self, url: str) -> str:
        data = {
            'reqtype': 'urlupload',
            'url': url
        }
        if self.userhash:
            data['userhash'] = self.userhash

        async with httpx.AsyncClient() as client:
            response = await client.post(CATBOX_API, data=data)

        if response.status_code == 200:
            return response.text.strip()
        else:
            raise Exception(f"URL upload failed: {response.status_code} {response.text}")

        

    async def delete_file(self, filename: str | list[str]) -> str:
        if not self.userhash:
            raise ValueError("Userhash is required to delete files")

        if isinstance(filename, list):
            files_str = " ".join(filename)
        else:
            files_str = filename

        data = {
            "reqtype": "deletefiles",
            "userhash": self.userhash,
            "files": files_str
        }

        async with httpx.AsyncClient() as client:
            response = await client.post(CATBOX_API, data=data)
        if response.status_code == 200:
            return response.text.strip()
        else:
            raise Exception(f"URL upload failed: {response.status_code} {response.text}")




from setuptools import setup, find_packages


setup(
    name='catboxpy',
    version='0.1.1',
    packages=find_packages(),
    install_requires=[
        "httpx>=0.24.0"
    ],
    description='A Python wrapper for the Catbox.moe and Litterbox api with Async features',
    long_description=open('README.md').read(),
    long_description_content_type="text/markdown",
    author='anshonweb',
    author_email='ansh.a.3112@gmail.com',
    url='https://github.com/anshonweb/catboxpy',
    license='MIT',
    keywords='catbox api python wrapper async litterbox',
)